FULL_NAME = 'Minecraft Server Creator'
SHORT_NAME = 'MCSC'
PACKAGE_NAME = 'mcservercreator'
DESCRIPTION = 'A tool to help you create and manage minecraft servers easier'
AUTHOR = 'Ivan1F'
VERSION = '1.0.1'
GITHUB_REPO = 'https://github.com/Ivan1F/MinecraftServerCreator'

VERSION_API = 'https://launchermeta.mojang.com/mc/game/version_manifest.json'
FABRIC_URL = 'https://maven.fabricmc.net/net/fabricmc/fabric-installer/0.7.3/fabric-installer-0.7.3.jar'

DEFAULT_SERVER_PROPERTIES = '''spawn-protection=16
max-tick-time=60000
query.port=25565
generator-settings=
sync-chunk-writes=true
force-gamemode=false
allow-nether=true
enforce-whitelist=false
gamemode=survival
broadcast-console-to-ops=true
enable-query=false
player-idle-timeout=0
text-filtering-config=
difficulty=easy
broadcast-rcon-to-ops=true
spawn-monsters=true
op-permission-level=4
pvp=true
entity-broadcast-range-percentage=100
snooper-enabled=true
level-type=default
enable-status=true
resource-pack-prompt=
hardcore=false
enable-command-block=false
network-compression-threshold=256
max-players=20
max-world-size=29999984
resource-pack-sha1=
function-permission-level=2
rcon.port=25575
server-port=25565
server-ip=
spawn-npcs=true
require-resource-pack=false
allow-flight=false
level-name=world
view-distance=10
resource-pack=
spawn-animals=true
white-list=false
rcon.password=
generate-structures=true
online-mode=true
level-seed=
prevent-proxy-connections=false
use-native-transport=true
enable-jmx-monitoring=false
motd=A Minecraft Server
rate-limit=0
enable-rcon=false
'''


"""
Entry for MCSC
"""

import sys

import inquirer
import os
from pip._internal.cli.main import main as pipmain

from mcservercreator import constants
from mcservercreator.utils import tools
from mcservercreator.utils import more_options
from mcservercreator.utils.properties_parser import Properties

info = '''
{0} {1} by Ivan1F
Thanks for using {0}!
{2} is open source, u can find it here: {3}
{2} is still under development, it may not work well
If u find any bug, please report it in the github issue
'''.format(constants.FULL_NAME, constants.VERSION, constants.SHORT_NAME, constants.GITHUB_REPO)

def print_info():
    print(info)

questions = [
    inquirer.List('mcversion', message='Input the version of the minecraft server core', choices=tools.get_all_versions_id()),
    inquirer.Confirm('fabric', message='Do u want to install fabric', default=True),
    inquirer.Confirm('mcdr', message='Do u want to install MCDReforged', default=True),
    inquirer.Checkbox('more', message="More options", choices=more_options.ALL)
]

def main():
    print_info()

    here = os.getcwd()

    if len(os.listdir(here)) != 0:
        ans = inquirer.prompt([inquirer.Confirm('confirm', message='The current directory is not empty, do u want to continue')])
        if not ans['confirm']:
            exit(0)

    # Show questions
    answers = inquirer.prompt(questions)

    more = answers['more']

    # More questions
    more_questions = []
    if more_options.CONFIG_PORT in more:
        more_questions.append(inquirer.Text('port', message='Input the server port'))
    if more_options.CONFIG_SEED in more:
        more_questions.append(inquirer.Text('seed', message='Input the world seed'))
    if more_options.CONFIG_DEFAULT_GAMEMODE in more:
        more_questions.append(inquirer.List('gamemode', message='Input the default gamemode', choices=['survival', 'creative', 'spectator', 'adventure']))
    if more_options.CONFIG_LEVEL_TYPE in more:
        more_questions.append(inquirer.List('level_type', message='Select the level type', choices=['default', 'flat', 'largebiomes', 'amplified', 'buffet']))
    if more_options.CONFIG_DIFFICULTLY in more:
        more_questions.append(inquirer.List('difficulty', message='Select the difficulty', choices=['easy', 'hard', 'normal', 'peaceful']))

    more_questions_ans = inquirer.prompt(more_questions)

    # Download MCDReforged
    if answers['mcdr']:
        if 'mcdreforged' not in tools.get_all_installed_packages():
            print('MCDReforged not found, installing it using pip...')
            pipmain(['install', 'mcdreforged'])
        else:
            print('Running MCDReforged to generate files of MCDR...')
            os.system('python -m mcdreforged')

    # Download vanilla server core
    tools.download_server_jar(os.path.join(here, 'server', 'server.jar'), answers['mcversion'])

    # Download and install Fabric
    tools.download_fabric_installer(os.path.join(here, 'server', 'fabric-installer.jar'))
    print('Installing Fabric Loader...')
    os.chdir(os.path.join(here, 'server'))
    cmd = 'java -jar {} server -mcversion {} nogui'.format('fabric-installer.jar', answers['mcversion'])
    # print(cmd, os.getcwd())
    os.system(cmd)
    os.chdir(here)

    print('Copying default server.properties to the server folder...')
    with open(os.path.join(here, 'server', 'server.properties'), 'w+') as f:
        f.write(constants.DEFAULT_SERVER_PROPERTIES)

    # Edit MCDR config file
    print('Editing MCDR config file...')
    mcdr_config_file = os.path.join(here, 'config.yml')
    if answers['fabric']:
        tools.replace_in_file(mcdr_config_file, 'minecraft_server', 'fabric-server-launch')  # Replace server core file in run command
    else:
        tools.replace_in_file(mcdr_config_file, 'minecraft_server', 'server')    # Replace server core file in run command

    if more_options.MCDR_LANG_ZH_CN in more:
        tools.replace_in_file(mcdr_config_file, 'en_us', 'zh_cn')

    # EULA
    print('Creating eula.txt')
    if more_options.AGREE_TO_EULA in more:
        with open(os.path.join(here, 'server', 'eula.txt'), 'w+') as f:
            f.write('eula=true')
            f.close()

    # Edit Minecraft config file
    print('Editing Minecraft config file...')
    server_properties_file = os.path.join(here, 'server', 'server.properties')
    properties = Properties(server_properties_file)
    if more_options.ALLOW_COMMAND_BLOCKS in more:
        properties.put('enable-command-block', 'true')
    if more_options.DISABLE_ONLINE_MODE in more:
        properties.put('online-mode', 'false')
    if more_options.CONFIG_PORT in more:
        properties.put('server-port', more_questions_ans['port'])
    if more_options.CONFIG_SEED in more:
        properties.put('level-seed', more_questions_ans['seed'])
    if more_options.CONFIG_DEFAULT_GAMEMODE in more:
        properties.put('gamemode', more_questions_ans['gamemode'])
    if more_options.CONFIG_LEVEL_TYPE in more:
        properties.put('level-type', more_questions_ans['level_type'])
    if more_options.CONFIG_DIFFICULTLY in more:
        properties.put('difficulty', more_questions_ans['difficulty'])

    properties.save()

    # Done
    print('Your server is ready! 🎉')
    ans = inquirer.prompt([inquirer.Confirm('run', message='Do u want to start now?', default=True)])
    if ans['run']:
        os.system('python -m mcdreforged')


if __name__ == '__main__':
    sys.exit(main())


import requests
import json
from progress.bar import Bar
import pkg_resources

from mcservercreator import constants

def get_all_versions_info():
    return json.loads(requests.get(constants.VERSION_API).text)

# ['20w46a', '20w45a', '1.16.4', '1.16.4-rc1', '1.16.4-pre2', ...]
def get_all_versions_id():
    ret = []
    versions_info = get_all_versions_info()
    for version in versions_info['versions']:
        ret.append(version['id'])
    return ret

# {'id': '1.16.5', 'type': 'release', 'url': 'https://launchermeta.mojang.com/v1/packages/436877ffaef948954053e1a78a366b8b7c204a91/1.16.5.json', 'time': '2021-01-14T16:09:14+00:00', 'releaseTime': '2021-01-14T16:05:32+00:00'}
def get_version_info(version: str):
    versions_info = get_all_versions_info()
    for version_info in versions_info['versions']:
        if version_info['id'] == version:
            return version_info

# 'https://launchermeta.mojang.com/v1/packages/436877ffaef948954053e1a78a366b8b7c204a91/1.16.5.json'
def get_version_url(version: str):
    version_info = get_version_info(version)
    return version_info['url']


def get_version_detail(version: str):
    url = get_version_url(version)
    return json.loads(requests.get(url).text)

# 'https://launcher.mojang.com/v1/objects/1b557e7b033b583cd9f66746b7a9ab1ec1673ced/server.jar'
def get_server_jar_url(version: str):
    detail = get_version_detail(version)
    return detail['downloads']['server']['url']

def download_server_jar(path: str, mc_version: str):
    with open(path, "wb") as f:
        response = requests.get(get_server_jar_url(mc_version), stream=True)
        total_length = response.headers.get('content-length')
        total_length = int(total_length)
        with Bar('Downloading Minecraft server core', max=total_length) as bar:
            dl = 0
            for data in response.iter_content(chunk_size=4096):
                dl += len(data)
                f.write(data)
                done = int(50 * dl / total_length)
                bar.next(len(data))

def download_fabric_installer(path: str):
    with open(path, "wb") as f:
        response = requests.get(constants.FABRIC_URL, stream=True)
        total_length = response.headers.get('content-length')
        total_length = int(total_length)
        with Bar('Downloading fabric installer', max=total_length) as bar:
            dl = 0
            for data in response.iter_content(chunk_size=4096):
                dl += len(data)
                f.write(data)
                done = int(50 * dl / total_length)
                bar.next(len(data))

def get_all_installed_packages():
    installed_packages = pkg_resources.working_set
    packages_list = sorted([i.key for i in installed_packages])
    return packages_list

def replace_in_file(path: str, old: str, new: str):
    ret = ''
    with open(path, 'r', encoding='utf-8') as f:
        for line in f:
            if old in line:
                line = line.replace(old, new)
            ret += line
    with open(path, 'w', encoding='utf-8') as f:
        f.write(ret)
        f.close()


class Properties:
    def __init__(self, file_name):
        self.file_name = file_name
        self.properties = {}

        with open(self.file_name, 'r') as f:
            for line in f:
                line = line.strip()
                if line.find('=') > 0 and not line.startswith('#'):
                    parsed = line.split('=')
                    self.properties[parsed[0].strip()] = parsed[1].strip()
            f.close()

    def has_key(self, key):
        return key in self.properties

    def get(self, key, default_value=''):
        if key in self.properties:
            return self.properties[key]
        return default_value

    def put(self, key, value):
        self.properties[key] = value

    def save(self):
        with open(self.file_name, 'w') as f:
            content = ''
            for p in self.properties.items():
                content += f'{p[0]}={p[1]}\n'
            f.write(content)
            f.close()


from setuptools import setup, find_packages
import os
import re

from mcservercreator import constants

# python setup.py sdist bdist_wheel
# python -m twine upload --repository testpypi dist/*
# python -m twine upload dist/*

here = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(here, 'requirements.txt')) as f:
    REQUIRED = [re.match(r'^[A-Za-z.]+', line).group() for line in f.readlines() if not len(line.strip()) == 0]

# print('REQUIRED = {}'.format(REQUIRED))

with open(os.path.join(here, 'README.md'), encoding='utf8') as f:
    LONG_DESCRIPTION = f.read()

setup(
    name=constants.PACKAGE_NAME,
    version=constants.VERSION,
    description=constants.DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    author=constants.AUTHOR,
    python_requires='>=3.6.0',
    url=constants.GITHUB_REPO,
    packages=find_packages(),
    install_requires=REQUIRED,
    include_package_data=True,
    license='GPL-3.0',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ]
)